@@ -6,30 +6,31 @@
     :synopsis: module that contains request handlers.
 """
 
-import re
-import os
 import json
 import logging
+import os
+import re
 import urllib
+import socket
+import struct
 from typing import (
     Union,
     Optional
 )
 
-import tornado.web
 import jsonschema
-from tornado.web import HTTPError
+import tornado.web
 from tornado.concurrent import Future
+from tornado.web import HTTPError
 
 import mockintosh
 from mockintosh.constants import PROGRAM, SUPPORTED_ENGINES, PYBARS, JINJA, SPECIAL_CONTEXT
 from mockintosh.exceptions import UnsupportedTemplateEngine
-from mockintosh.templating import TemplateRenderer
-from mockintosh.params import PathParam, HeaderParam, QueryStringParam, BodyParam
+from mockintosh.hbs.methods import Random as hbs_Random, Date as hbs_Date
+from mockintosh.j2.methods import Random as j2_Random, Date as j2_Date
 from mockintosh.methods import _safe_path_split, _detect_engine
-
-from mockintosh.hbs.methods import Random as hbs_Random
-from mockintosh.j2.methods import Random as j2_Random
+from mockintosh.params import PathParam, HeaderParam, QueryStringParam, BodyParam
+from mockintosh.templating import TemplateRenderer
 
 OPTIONS = 'options'
 ORIGIN = 'Origin'
@@ -39,6 +40,9 @@ NON_PREFLIGHT_METHODS = ("GET", "HEAD", "POST", "DELETE", "PATCH", "PUT")
 hbs_random = hbs_Random()
 j2_random = j2_Random()
 
+hbs_date = hbs_Date()
+j2_date = j2_Date()
+
 counters = {}
 
 
@@ -140,7 +144,7 @@ class GenericHandler(tornado.web.RequestHandler):
             self._unimplemented_method()
 
     def log_request(self):
-        logging.debug('Received request:\n%s' % self.request.__dict__)
+        logging.debug('Received request:\n%s', self.request.__dict__)
 
     def add_params(self, context):
         if not hasattr(self, 'custom_params'):
@@ -169,11 +173,11 @@ class GenericHandler(tornado.web.RequestHandler):
             if template_path is None:
                 return None
             with open(template_path, 'rb') as file:
-                logging.info('Reading external file from path: %s' % template_path)
+                logging.debug('Reading external file from path: %s', template_path)
                 source_text = file.read()
                 try:
                     source_text = source_text.decode('utf-8')
-                    logging.debug('Template file text: %s' % source_text)
+                    logging.debug('Template file text: %s', source_text)
                 except UnicodeDecodeError:
                     is_binary = True
                     logging.debug('Template file is binary. Templating disabled.')
@@ -183,32 +187,11 @@ class GenericHandler(tornado.web.RequestHandler):
         if is_binary or not self.custom_response.get('useTemplating', True):
             compiled = source_text
         else:
-            if template_engine == PYBARS:
-                from mockintosh.hbs.methods import fake, counter, json_path
-                self.custom_context['random'] = hbs_random
-            elif template_engine == JINJA:
-                from mockintosh.j2.methods import fake, counter, json_path
-                self.custom_context['random'] = j2_random
-            else:
-                raise UnsupportedTemplateEngine(template_engine, SUPPORTED_ENGINES)
-
-            renderer = TemplateRenderer(
-                template_engine,
-                source_text,
-                inject_objects=self.custom_context,
-                inject_methods=[
-                    fake,
-                    counter,
-                    json_path
-                ],
-                add_params_callback=self.add_params,
-                fill_undefineds=True
-            )
-            compiled, context = renderer.render()
+            compiled, context = self.common_template_renderer(template_engine, source_text)
             self.populate_counters(context)
 
         if not is_binary:
-            logging.debug('Render output: %s' % compiled)
+            logging.debug('Render output: %s', compiled)
 
         return compiled
 
@@ -277,31 +260,31 @@ class GenericHandler(tornado.web.RequestHandler):
         status_code = None
         if 'status' in self.custom_response:
             if isinstance(self.custom_response['status'], str):
-                if self.definition_engine == PYBARS:
-                    from mockintosh.hbs.methods import counter
-                elif self.definition_engine == JINJA:
-                    from mockintosh.j2.methods import counter
-                else:
-                    raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
-
-                renderer = TemplateRenderer(
+                compiled, context = self.common_template_renderer(
                     self.definition_engine,
-                    self.custom_response['status'],
-                    inject_objects=self.custom_context,
-                    inject_methods=[
-                        counter
-                    ],
-                    add_params_callback=self.add_params,
-                    fill_undefineds=True
+                    self.custom_response['status']
                 )
-                compiled, context = renderer.render()
                 self.populate_counters(context)
-                status_code = int(compiled)
+                try:
+                    status_code = int(compiled)
+                except ValueError:
+                    status_code = compiled
             else:
                 status_code = self.custom_response['status']
         else:
             status_code = 200
-        self.set_status(status_code)
+
+        if isinstance(status_code, str) and status_code.lower() == 'rst':
+            self.request.server_connection.stream.socket.setsockopt(
+                socket.SOL_SOCKET,
+                socket.SO_LINGER,
+                struct.pack('ii', 1, 0)
+            )
+            self.request.server_connection.stream.close()
+        if isinstance(status_code, str) and status_code.lower() == 'fin':
+            self.request.server_connection.stream.close()
+        else:
+            self.set_status(status_code)
 
     def analyze_headers(self):
         if SPECIAL_CONTEXT not in self.initial_context or 'headers' not in self.initial_context[SPECIAL_CONTEXT]:
@@ -354,13 +337,6 @@ class GenericHandler(tornado.web.RequestHandler):
         if 'headers' not in self.custom_response:
             return
 
-        if self.definition_engine == PYBARS:
-            from mockintosh.hbs.methods import counter
-        elif self.definition_engine == JINJA:
-            from mockintosh.j2.methods import counter
-        else:
-            raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
-
         for key, value in self.custom_response['headers'].items():
             value_list = None
             if isinstance(value, list):
@@ -371,17 +347,7 @@ class GenericHandler(tornado.web.RequestHandler):
 
             new_value_list = []
             for value in value_list:
-                renderer = TemplateRenderer(
-                    self.definition_engine,
-                    value,
-                    inject_objects=self.custom_context,
-                    inject_methods=[
-                        counter
-                    ],
-                    add_params_callback=self.add_params,
-                    fill_undefineds=True
-                )
-                new_value, context = renderer.render()
+                new_value, context = self.common_template_renderer(self.definition_engine, value)
                 self.populate_counters(context)
                 new_value_list.append(new_value)
 
@@ -456,9 +422,9 @@ class GenericHandler(tornado.web.RequestHandler):
                     if isinstance(json_schema, str) and len(json_schema) > 1 and json_schema[0] == '@':
                         json_schema_path = self.resolve_relative_path(json_schema)
                         with open(json_schema_path, 'r') as file:
-                            logging.info('Reading JSON schema file from path: %s' % json_schema_path)
+                            logging.info('Reading JSON schema file from path: %s', json_schema_path)
                             json_schema = json.load(file)
-                            logging.debug('JSON schema: %s' % json_schema)
+                            logging.debug('JSON schema: %s', json_schema)
                     json_data = None
 
                     if body and json_schema:
@@ -548,24 +514,7 @@ class GenericHandler(tornado.web.RequestHandler):
         relative_path = None
         orig_relative_path = source_text[1:]
 
-        if self.definition_engine == PYBARS:
-            from mockintosh.hbs.methods import counter
-        elif self.definition_engine == JINJA:
-            from mockintosh.j2.methods import counter
-        else:
-            raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
-
-        renderer = TemplateRenderer(
-            self.definition_engine,
-            orig_relative_path,
-            inject_objects=self.custom_context,
-            inject_methods=[
-                counter
-            ],
-            add_params_callback=self.add_params,
-            fill_undefineds=True
-        )
-        orig_relative_path, context = renderer.render()
+        orig_relative_path, context = self.common_template_renderer(self.definition_engine, orig_relative_path)
         self.populate_counters(context)
 
         error_msg = 'External template file \'%s\' couldn\'t be accessed or found!' % orig_relative_path
@@ -615,9 +564,7 @@ class GenericHandler(tornado.web.RequestHandler):
         self.set_cors_headers()
 
     def should_cors(self):
-        return not self.__class__.__name__ == 'ErrorHandler' and (
-            self.is_options and self.request.method.lower() not in self.methods.keys()
-        )
+        return self.is_options and self.request.method.lower() not in self.methods.keys()
 
     def load_dataset(self, dataset):
         if isinstance(dataset, list):
@@ -625,9 +572,9 @@ class GenericHandler(tornado.web.RequestHandler):
         else:
             dataset_path = self.resolve_relative_path(dataset)
             with open(dataset_path, 'r') as file:
-                logging.info('Reading dataset file from path: %s' % dataset_path)
+                logging.info('Reading dataset file from path: %s', dataset_path)
                 data = json.load(file)
-                logging.debug('Dataset: %s' % data)
+                logging.debug('Dataset: %s', data)
                 return data
 
     def loop_alternative(self, alternative, key, subkey):
@@ -647,6 +594,33 @@ class GenericHandler(tornado.web.RequestHandler):
                 return
         return alternative[key][alternative[index_key]]
 
+    def common_template_renderer(self, template_engine, text):
+        if template_engine == PYBARS:
+            from mockintosh.hbs.methods import fake, counter, json_path, escape_html
+            self.custom_context['random'] = hbs_random
+            self.custom_context['date'] = hbs_date
+        elif template_engine == JINJA:
+            from mockintosh.j2.methods import fake, counter, json_path, escape_html
+            self.custom_context['random'] = j2_random
+            self.custom_context['date'] = j2_date
+        else:
+            raise UnsupportedTemplateEngine(template_engine, SUPPORTED_ENGINES)
+
+        renderer = TemplateRenderer(
+            template_engine,
+            text,
+            inject_objects=self.custom_context,
+            inject_methods=[
+                fake,
+                counter,
+                json_path,
+                escape_html
+            ],
+            add_params_callback=self.add_params,
+            fill_undefineds=True
+        )
+        return renderer.render()
+
 
 class NotParsedJSON():
     pass
@@ -676,7 +650,7 @@ class Request():
             try:
                 self._json = json.loads(self.body)
             except json.JSONDecodeError:
-                logging.warning('Failed to decode request body to JSON:\n%s' % self.body)
+                logging.warning('Failed to decode request body to JSON:\n%s', self.body)
                 self._json = None
         return self._json
 
