@@ -6,25 +6,56 @@
     :synopsis: module that server classes.
 """
 
-import sys
 import logging
+import sys
+from abc import abstractmethod
 from collections import OrderedDict
+from os import path
 
 import tornado.ioloop
 import tornado.web
 from tornado.routing import Rule, RuleRouter, HostMatches
 
+from mockintosh.exceptions import CertificateLoadingError
 from mockintosh.handlers import GenericHandler
 from mockintosh.overrides import Application
 
+__location__ = path.abspath(path.dirname(__file__))
 
-class HttpServer():
 
-    def __init__(self, definition, debug=False, interceptors=()):
+class Impl:
+    @abstractmethod
+    def get_server(self, app, is_ssl, ssl_options):
+        pass
+
+    @abstractmethod
+    def serve(self):
+        pass
+
+
+class TornadoImpl(Impl):
+    def get_server(self, router, is_ssl, ssl_options):
+        if is_ssl:
+            server = tornado.web.HTTPServer(router, ssl_options=ssl_options)
+        else:
+            server = tornado.web.HTTPServer(router)
+
+        return server
+
+    def serve(self):
+        tornado.ioloop.IOLoop.current().start()
+
+
+class HttpServer:
+
+    def __init__(self, definition, impl: Impl, debug=False, interceptors=(), address='', services_list=[]):
         self.definition = definition
+        self.impl = impl
+        self.address = address
         self.globals = self.definition.data['globals'] if 'globals' in self.definition.data else {}
         self.debug = debug
         self.interceptors = interceptors
+        self.services_list = services_list
         self.services_log = []
         self.load()
 
@@ -38,42 +69,70 @@ class HttpServer():
 
         for port, services in port_mapping.items():
             rules = []
+            ssl = False
+            cert_file = path.join(__location__, 'ssl', 'cert.pem')
+            key_file = path.join(__location__, 'ssl', 'key.pem')
             for service in services:
+                ssl = service.get('ssl', False)
+                if ssl:
+                    if 'sslCertFile' in service:
+                        cert_file = self.resolve_cert_path(service['sslCertFile'])
+                    if 'sslKeyFile' in service:
+                        key_file = self.resolve_cert_path(service['sslKeyFile'])
+                    break
+
+            protocol = 'https' if ssl else 'http'
+            ssl_options = {
+                "certfile": cert_file,
+                "keyfile": key_file,
+            }
+
+            for service in services:
+                if self.services_list:
+                    if 'name' in service:
+                        if service['name'] not in self.services_list:
+                            continue
+                    else:
+                        continue
+
                 endpoints = []
                 if 'endpoints' in service:
                     endpoints = self.merge_alternatives(service['endpoints'])
-                app = self.make_app(endpoints, self.globals, self.debug)
+                app = self.make_app(endpoints, self.globals, debug=self.debug)
                 if 'hostname' not in service:
-                    app.listen(service['port'])
-                    logging.info('Will listen port number: %d' % service['port'])
-                    self.services_log.append('Serving at http://%s:%s%s' % (
+                    server = self.impl.get_server(app, ssl, ssl_options)
+                    server.listen(service['port'], address=self.address)
+                    logging.debug('Will listen port number: %d' % service['port'])
+                    self.services_log.append('Serving at %s://%s:%s%s' % (
+                        protocol,
                         'localhost',
                         service['port'],
-                        ' the mock for %r' % service['comment'] if 'comment' in service else ''
+                        ' the mock for %r' % service['name'] if 'name' in service else ''
                     ))
                 else:
                     rules.append(
                         Rule(HostMatches(service['hostname']), app)
                     )
 
-                    logging.info('Registered hostname and port: %s://%s:%d' % (
-                        'http',
+                    logging.debug('Registered hostname and port: %s://%s:%d' % (
+                        protocol,
                         service['hostname'],
                         service['port']
                     ))
-                    self.services_log.append('Serving at http://%s:%s%s' % (
+                    self.services_log.append('Serving at %s://%s:%s%s' % (
+                        protocol,
                         service['hostname'],
                         service['port'],
-                        ' the mock for %r' % service['comment'] if 'comment' in service else ''
+                        ' the mock for %r' % service['name'] if 'name' in service else ''
                     ))
-                if 'comment' in service:
-                    logging.info('Finished registering: %s' % service['comment'])
+                if 'name' in service:
+                    logging.debug('Finished registering: %s' % service['name'])
 
             if rules:
                 router = RuleRouter(rules)
-                server = tornado.web.HTTPServer(router)
-                server.listen(services[0]['port'])
-                logging.info('Will listen port number: %d' % service['port'])
+                server = self.impl.get_server(router, ssl, ssl_options)
+                server.listen(services[0]['port'], address=self.address)
+                logging.debug('Will listen port number: %d' % service['port'])
 
     def merge_alternatives(self, endpoints):
         new_endpoints = {}
@@ -88,6 +147,8 @@ class HttpServer():
                 extracted_parts[key] = endpoint[key]
                 if 'id' not in extracted_parts:
                     extracted_parts['id'] = None
+                if 'counters' not in extracted_parts:
+                    extracted_parts['counters'] = {}
             if identifier not in new_endpoints:
                 new_endpoints[identifier] = {}
                 new_endpoints[identifier]['path'] = endpoint['path']
@@ -110,7 +171,7 @@ class HttpServer():
             logging.info(service_log)
 
         logging.info('Mock server is ready!')
-        tornado.ioloop.IOLoop.current().start()
+        self.impl.serve()
 
     def make_app(self, endpoints, _globals, debug=False):
         endpoint_handlers = []
@@ -131,6 +192,16 @@ class HttpServer():
                 )
             )
             for method, alternatives in endpoint['methods'].items():
-                logging.info('Registered endpoint: %s %s' % (method.upper(), endpoint['path']))
+                logging.debug('Registered endpoint: %s %s' % (method.upper(), endpoint['path']))
                 logging.debug('with alternatives:\n%s' % alternatives)
         return Application(endpoint_handlers, debug=debug, interceptors=self.interceptors)
+
+    def resolve_cert_path(self, cert_path):
+        relative_path = path.join(self.definition.source_dir, cert_path)
+        if not path.isfile(relative_path):
+            raise CertificateLoadingError('File not found on path `%s`' % cert_path)
+        relative_path = path.abspath(relative_path)
+        if not relative_path.startswith(self.definition.source_dir):
+            raise CertificateLoadingError('Path `%s` is inaccessible!' % cert_path)
+
+        return relative_path
